"""
hudai.resources.article_id
"""
from ..helpers.resource import Resource


class CollateralResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/organizations/{organization_id}/collateral')
        self.resource_name = 'Collateral'

    def list(self, organization_id, page=None):
        query_params = self._set_limit_offset({'page': page})

        return self.http_get('/', params={'organization_id': organization_id},
                                  query_params=query_params)

    def metadata(self, organization_id):
        return self.http_get('/metadata',
                             params={'organization_id': organization_id})

    def create(self, organization_id,
               name=None,
               description=None,
               content_url=None,
               plaintext_url=None,
               filetype=None,
               size=None,
               data_science_metadata=None):
        return self.http_post('/', params={'organization_id': organization_id},
                              data={
                                  'name': name,
                                  'description': description,
                                  'content_url': content_url,
                                  'plaintext_url': plaintext_url,
                                  'filetype': filetype,
                                  'size': size,
                                  'data_science_metadata': data_science_metadata
                              })

    def fetch(self, organization_id, entity_id):
        return self.http_get('/{id}',
                             params={
                                 'organization_id': organization_id,
                                 'id': entity_id
                             })

    def update(self, organization_id, entity_id,
               name=None,
               description=None,
               content_url=None,
               plaintext_url=None,
               filetype=None,
               size=None,
               data_science_metadata=None):
        return self.http_patch('/{id}',
                               params={
                                   'organization_id': organization_id,
                                   'id': entity_id
                               },
                               data={
                                   'name': name,
                                   'description': description,
                                   'content_url': content_url,
                                   'plaintext_url': plaintext_url,
                                   'filetype': filetype,
                                   'size': size,
                                   'data_science_metadata': data_science_metadata
                               })

    def delete(self, organization_id, entity_id):
        return self.http_delete('/{id}',
                                params={
                                    'organization_id': organization_id,
                                    'id': entity_id
                                })



"""
hudai.resources.company_industry
"""
from ..helpers.resource import Resource


class CompanyIndustryResource(Resource):
    def __init__(self, client):
        Resource.__init__(
            self, client, base_path='/companies/{company_id}/industries')
        self.resource_name = 'CompanyIndustry'

    def list(self, company_id, page=None):
        query_params = self._set_limit_offset({'page': page})

        return self.http_get('/', params={'company_id': company_id},
                             query_params=query_params)

    def create(self, company_id, industry_id):
        return self.http_post('/', params={'company_id': company_id},
                              data={'industry_id': industry_id})

    def fetch(self, company_id, industry_id):
        return self.http_get('/{industry_id}',
                             params={'company_id': company_id, 'industry_id': industry_id})

    def delete(self, company_id, industry_id):
        return self.http_delete('/{industry_id}',
                                params={'company_id': company_id, 'industry_id': industry_id})



"""
hudai.resources.person_quote
"""
from ..helpers.resource import Resource


class PersonQuoteResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/people/quotes')
        self.resource_name = 'PersonQuote'

    def list(self, person_id, article_id=None, term=None, min_importance=None, page=None):
        query_params = self._set_limit_offset({
            'article_id': article_id,
            'person_id': person_id,
            'term': term,
            'min_importance': min_importance,
            'page': page
        })

        return self.http_get('/', query_params=query_params)

    def create(self, person_id, article_id, term, text, importance_score=None):
        return self.http_post('/',
                              data={
                                  'article_id': article_id,
                                  'person_id': person_id,
                                  'term': term,
                                  'text': text,
                                  'importance_score': importance_score
                              })

    def fetch(self, quote_id):
        return self.http_get('/{id}', params={'id': quote_id})

    def delete(self, quote_id):
        return self.http_delete('/{id}', params={'id': quote_id})



"""
hudai.resources.user_company_group
"""
from ..helpers.resource import Resource


class UserCompanyGroupResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/companies/groups')
        self.resource_name = 'UserCompanyGroup'

    def list(self, user_id=None, name=None, page=None):
        return self._list(user_id=user_id, name=name, page=page)

    def create(self, user_id=None, name=None):
        return self._create(user_id=user_id, name=name)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, user_id=None, name=None):
        return self._update(entity_id, user_id=user_id, name=name)

    def delete(self, entity_id):
        return self._delete(entity_id)

    def add_company(self, group_id, company_id):
        return self.http_post('/{group_id}/companies',
                                params={'group_id': group_id},
                                data={'company_id': company_id})

    def remove_company(self, group_id, company_id):
        return self.http_delete('/{group_id}/companies',
                                params={'group_id': group_id},
                                data={'company_id': company_id})



"""
hudai.resources.company_key_term
"""
from ..helpers.resource import Resource


class CompanyEventResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/companies/events')
        self.resource_name = 'CompanyEvent'

    def list(self,
             company_id=None,
             starting_before=None,
             starting_after=None,
             ending_before=None,
             ending_after=None,
             occurring_at=None,
             title=None,
             event_type=None,
             page=None):
        query_params = self._set_limit_offset({
            'company_id': company_id,
            'starting_before': starting_before,
            'starting_after': starting_after,
            'ending_before': ending_before,
            'ending_after': ending_after,
            'occurring_at': occurring_at,
            'title': title,
            'type': event_type,
            'page': page
        })

        return self.http_get('/', query_params=query_params)

    def create(self,
               company_id=None,
               title=None,
               description=None,
               event_type=None,
               link_url=None,
               starts_at=None,
               ends_at=None):
        return self.http_post('/',
                              data={'company_id': company_id,
                                    'title': title,
                                    'description': description,
                                    'type': event_type,
                                    'link_url': link_url,
                                    'starts_at': starts_at,
                                    'ends_at': ends_at})

    def fetch(self, entity_id):
        return self.http_get('/{id}', params={'id': entity_id})

    def update(self, entity_id,
               company_id=None,
               title=None,
               description=None,
               event_type=None,
               link_url=None,
               starts_at=None,
               ends_at=None):
        return self.http_put('/{id}',
                             params={'id': entity_id},
                             data={'company_id': company_id,
                                   'title': title,
                                   'description': description,
                                   'type': event_type,
                                   'link_url': link_url,
                                   'starts_at': starts_at,
                                   'ends_at': ends_at})


    def delete(self, entity_id):
        return self.http_delete('/{id}', params={'id': entity_id})



"""
hudai.resources.company_profile
"""
from ..helpers.resource import Resource


class CompanyProfileResource(Resource):
    def __init__(self, client):
        Resource.__init__(
            self, client, base_path='/companies/{company_id}/profiles')
        self.resource_name = 'CompanyProfile'

    def fetch(self, company_id):
        return self.http_get(
            '/',
            params={
                'company_id': company_id
            }
        )



"""
Python wrapper for HUD.ai data access layer

Example usage:

from hudai.client import Client as HudAiClient

client = HudAiClient()
"""

__package_name__ = 'hudai'
__description__ = ('HUD.ai API python bindings')
__keywords__ = ['hud.ai', 'hudai', 'foundry.ai', 'foundrydc']
__url__ = 'https://github.com/FoundryAI/hud-ai-python'
__version__ = '2.25.0'
__author__ = 'HUD.ai Engineering'
__email__ = 'engineering@hud.ai'
__license__ = 'MIT'

from .client import Client



"""
hudai.resources.stock_alert
"""
from ..helpers.resource import Resource


class StockAlertResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/companies/stock-alerts')
        self.resource_name = 'StockAlert'

    def list(self,
             company_id=None,
             occurring_before=None,
             occurring_after=None,
             page=None):
        query_params = self._set_limit_offset({
            'company_id': company_id,
            'occurring_before': occurring_before,
            'occurring_after': occurring_after,
            'page': page
        })

        return self.http_get('/', query_params=query_params)

    def create(self,
               company_id=None,
               first_value=None,
               first_value_occurred_at=None,
               second_value=None,
               second_value_occurred_at=None):
        return self.http_post('/',
                              data={'company_id': company_id,
                                    'first_value': first_value,
                                    'first_value_occurred_at': first_value_occurred_at,
                                    'second_value': second_value,
                                    'second_value_occurred_at': second_value_occurred_at})

    def fetch(self, entity_id):
        return self.http_get('/{id}', params={'id': entity_id})

    def delete(self, entity_id):
        return self.http_delete('/{id}', params={'id': entity_id})



"""
hudai.resources.article_id
"""
from ..helpers.resource import Resource


class RelevantArticleCollateralResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/relevant/{relevant_article_id}/collateral')
        self.resource_name = 'RelevantArticleCollateral'

    def list(self, relevant_article_id, collateral_id=None, page=None):
        query_params = self._set_limit_offset({
            'collateral_id': collateral_id,
            'page': page
        })

        return self.http_get('/', params={'relevant_article_id': relevant_article_id},
                                  query_params=query_params)

    def create(self, relevant_article_id, collateral_id):
        return self.http_post('/', params={'relevant_article_id': relevant_article_id},
                                   data={ 'collateral_id': collateral_id })

    def fetch(self, relevant_article_id, collateral_id):
        return self.http_get('/{id}',
                             params={
                                 'relevant_article_id': relevant_article_id,
                                 'id': collateral_id
                             })

    def delete(self, relevant_article_id, collateral_id):
        return self.http_delete('/{id}',
                                params={
                                    'relevant_article_id': relevant_article_id,
                                    'id': collateral_id
                                })



"""
hudai.resources
"""

from .article import ArticleResource
from .article_company import ArticleCompanyResource
from .article_highlights import ArticleHighlightsResource
from .article_key_term import ArticleKeyTermResource
from .article_tag import ArticleTagResource
from .collateral import CollateralResource
from .company import CompanyResource
from .company_event import CompanyEventResource
from .company_industry import CompanyIndustryResource
from .company_key_term import CompanyKeyTermResource
from .company_profile import CompanyProfileResource
from .domain import DomainResource
from .industry import IndustryResource
from .key_term import KeyTermResource
from .organization import OrganizationResource
from .person import PersonResource
from .person_key_term import PersonKeyTermResource
from .person_quote import PersonQuoteResource
from .relevant_article_collateral import RelevantArticleCollateralResource
from .relevant_articles import RelevantArticlesResource
from .source import SourceResource
from .stock_alert import StockAlertResource
from .system_event import SystemEventResource
from .system_task import SystemTaskResource
from .text_corpus import TextCorpusResource
from .tweets import TweetResource
from .user import UserResource
from .user_company import UserCompanyResource
from .user_company_group import UserCompanyGroupResource
from .user_contact import UserContactResource
from .user_digest_subscription import UserDigestSubscriptionResource
from .user_key_term import UserKeyTermResource
from .user_source import UserSourceResource


"""
hudai.resources.key_term
"""
from ..helpers.resource import Resource


class KeyTermResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/key-terms')
        self.resource_name = 'KeyTerm'

    def list(self, page=None):
        return self._list(page=page)

    def create(self, term=None):
        return self._create(term=term)

    def fetch(self, term):
        return self._fetch(term)

    def delete(self, term):
        return self._delete(term)

    def identify(self, text):
        return self.http_post('/identify', data={'text': text})



"""
hudai.resources.user_source
"""
from ..helpers.resource import Resource


class UserSourceResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/sources/users')
        self.resource_name = 'UserSource'

    def list(
            self,
            user_id=None,
            source_id=None,
            created_after=None,
            created_before=None,
            page=None
    ):
        return self._list(
            user_id=user_id,
            source_id=source_id,
            created_after=created_after,
            created_before=created_before,
            page=page
        )

    def create(self, user_id, source_id, reliability_score):
        return self._create(user_id=user_id, source_id=source_id, reliability_score=reliability_score)

    def update(self, user_id, source_id, reliability_score=None):
        return self.http_put('/', 
            data={
                'user_id': user_id,
                'source_id': source_id,
                'reliability_score': reliability_score
            })

    def delete(self, user_id, source_id):
        return self.http_delete('/', query_params={'user_id': user_id, 'source_id': source_id})


class HudAiError(Exception):
    def __init__(self, message=None, type='validation_error'):
        super(HudAiError, self).__init__(message)

        self._message = message
        self.type = type



"""
hudai.resources.text_corpus
"""
from ..helpers.resource import Resource


class TextCorpusResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/text-corpora')
        self.resource_name = 'TextCorpus'

    def list(self, user_id=None, corpus_type=None, page=None):
        return self._list(user_id=user_id, type=corpus_type, page=page)

    def create(self, user_id=None, corpus_type=None, body=None):
        return self._create(user_id=user_id, type=corpus_type, body=body)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, user_id=None, corpus_type=None, body=None):
        return self._update(entity_id,
                            user_id=user_id,
                            type=corpus_type,
                            body=body)

    def delete(self, entity_id):
        return self._delete(entity_id)



"""
hudai.resources.article_key_term
"""
from ..helpers.resource import Resource


class ArticleCompanyResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/companies')
        self.resource_name = 'ArticleCompany'

    def list(self, article_id=None, company_id=None, page=None):
        return self._list(
            article_id=article_id,
            company_id=company_id,
            page=page
        )

    def create(self, article_id, company_id):
        return self._create(
            article_id=article_id,
            company_id=company_id
        )

    def delete(self, article_id, company_id):
        return self.http_delete('/',
                                query_params={
                                    'article_id': article_id,
                                    'company_id': company_id
                                })



"""
hudai.resources.industry
"""
from ..helpers.resource import Resource


class IndustryResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/companies/industries')
        self.resource_name = 'Industry'

    def list(self, page=None):
        return self._list(page=page)

    def create(self,
               name=None,
               text_corpus=None):
        return self._create(name=name,
                            text_corpus=text_corpus)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id,
               name=None,
               text_corpus=None):
        return self._update(entity_id,
                            name=name,
                            text_corpus=text_corpus)

    def delete(self, entity_id):
        return self._delete(entity_id)
